use crate::maven::{
    items::{Item, ListItem},
    version::ComparableVersion,
};
use std::{borrow::Borrow, cmp::Ordering};
impl PartialEq for ListItem {
    fn eq(&self, other: &Self) -> bool {
        self.cmp(other).is_eq()
    }
}
impl Eq for ListItem {}
impl PartialOrd for ListItem {
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        Some(self.cmp(other))
    }
}
impl Ord for ListItem {
    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
        let mut left = self.iter();
        let mut right = other.iter();
        let null_value = Item::zero();
        loop {
            let l = left.next();
            let r = right.next();
            if l.is_none() && r.is_none() {
                break;
            }
            let result = match (l, r) {
                (None, None) => Ordering::Equal,
                (None, Some(y)) => null_value.cmp(y),
                (Some(x), None) => x.cmp(&null_value),
                (Some(x), Some(y)) => x.cmp(y),
            };
            if result.is_ne() {
                return result;
            }
        }
        Ordering::Equal
    }
}
impl PartialEq for Item {
    fn eq(&self, other: &Self) -> bool {
        self.cmp(other).is_eq()
    }
}
impl Eq for Item {}
impl PartialOrd for Item {
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        Some(self.cmp(other))
    }
}
impl Ord for Item {
    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
        if self.is_null() || other.is_null() {
            if !other.is_null() {
                return other.compare_null().reverse();
            } else if !self.is_null() {
                return self.compare_null();
            } else {
                return Ordering::Equal;
            }
        }
        match (self, other) {
            (Item::String(x), Item::String(y)) => x.cmp(y),
            (Item::List(x), Item::List(y)) => x.cmp(y),
            (Item::Integer(x), Item::Integer(y)) => x.cmp(y),
            (x, y) => {
                if x.ordinal() > y.ordinal() {
                    Ordering::Greater
                } else {
                    Ordering::Less
                }
            }
        }
    }
}
impl Item {
    fn compare_null(&self) -> Ordering {
        let empty_list = ListItem(vec![]);
        match self {
            Item::String(s) => s.ordinal().cmp(&0),
            Item::List(l) => l.cmp(&empty_list),
            Item::Integer(i) => i.cmp(&0),
        }
    }
}
impl PartialEq for ComparableVersion {
    fn eq(&self, other: &Self) -> bool {
        self.cmp(other).is_eq()
    }
}
impl Eq for ComparableVersion {}
impl PartialOrd for ComparableVersion {
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        Some(self.cmp(other))
    }
}
impl Ord for ComparableVersion {
    fn cmp(&self, other: &Self) -> Ordering {
        self.items.cmp(other.items.borrow())
    }
}
