#[derive(PartialEq, Eq, PartialOrd, Ord, Debug)]
pub enum Qualifier {
    Alpha,
    Beta,
    Milestone,
    Rc,
    Snapshot,
    Release,
    Sp,
    Unknown(String),
}
impl Qualifier {
    pub fn new(value: String) -> Self {
        match value.to_lowercase().as_str() {
            "alpha" => Self::Alpha,
            "beta" => Self::Beta,
            "milestone" => Self::Milestone,
            "rc" => Self::Rc,
            "cr" => Self::Rc,
            "snapshot" => Self::Snapshot,
            "" => Self::Release,
            "ga" => Self::Release,
            "final" => Self::Release,
            "sp" => Self::Sp,
            s => Self::Unknown(s.to_string()),
        }
    }
    pub fn is_release(&self) -> bool {
        matches!(self, Qualifier::Release)
    }
    pub fn ordinal(&self) -> i8 {
        match self {
            Qualifier::Alpha => -5,
            Qualifier::Beta => -4,
            Qualifier::Milestone => -3,
            Qualifier::Rc => -2,
            Qualifier::Snapshot => -1,
            Qualifier::Release => 0,
            Qualifier::Sp => 1,
            Qualifier::Unknown(_) => 2,
        }
    }
}
impl ToString for Qualifier {
    fn to_string(&self) -> String {
        match self {
            Qualifier::Alpha => "alpha".to_string(),
            Qualifier::Beta => "beta".to_string(),
            Qualifier::Milestone => "milestone".to_string(),
            Qualifier::Rc => "rc".to_string(),
            Qualifier::Snapshot => "snapshot".to_string(),
            Qualifier::Release => String::new(),
            Qualifier::Sp => "sp".to_string(),
            Qualifier::Unknown(s) => s.clone(),
        }
    }
}
impl Clone for Qualifier {
    fn clone(&self) -> Self {
        match self {
            Self::Alpha => Self::Alpha,
            Self::Beta => Self::Beta,
            Self::Milestone => Self::Milestone,
            Self::Rc => Self::Rc,
            Self::Snapshot => Self::Snapshot,
            Self::Release => Self::Release,
            Self::Sp => Self::Sp,
            Self::Unknown(arg0) => Self::Unknown(arg0.clone()),
        }
    }
}
