use crate::{errors::Result, maven::items::Item};
#[derive(Debug)]
pub struct ComparableVersion {
    pub(crate) value: String,
    pub(crate) items: Item,
}
impl ComparableVersion {
    //Parses Maven version string
    pub fn parse(value: String) -> Result<Self> {
        let canonical = Self::normalize(value.clone());
        let items = Item::parse_list(canonical);
        Ok(ComparableVersion { value, items })
    }
    //Converts Maven version string into it's canonical form
    pub fn normalize(version: String) -> String {
        let mut v = String::new();
        let mut buffer: Vec<char> = vec![];
        for c in version.chars() {
            if c == '.' || c == '-' {
                if buffer.is_empty() {
                    buffer.push('0');
                }
                for ch in buffer.iter() {
                    v.push(*ch);
                }
                buffer.clear();
                v.push(c);
            } else if buffer.is_empty() || (c.is_ascii_digit() == buffer[0].is_ascii_digit()) {
                buffer.push(c);
            } else {
                for ch in buffer.iter() {
                    v.push(*ch);
                }
                if buffer.len() == 1 {
                    let app = match buffer[0] {
                        'a' => "lpha",
                        'b' => "eta",
                        'm' => "ilestone",
                        _ => "",
                    };
                    v.push_str(app);
                }
                v.push('-');
                buffer.clear();
                buffer.push(c);
            }
        }
        if buffer.is_empty() {
            v.push('0');
        } else {
            for ch in buffer {
                v.push(ch);
            }
        }
        v.to_lowercase()
    }
    pub fn parsed_string(&self) -> String {
        self.items.to_string()
    }
}
impl ToString for ComparableVersion {
    fn to_string(&self) -> String {
        self.value.clone()
    }
}
impl Clone for ComparableVersion {
    fn clone(&self) -> Self {
        Self {
            value: self.value.clone(),
            items: self.items.clone(),
        }
    }
}
impl Default for ComparableVersion {
    fn default() -> Self {
        Self {
            value: "0".to_string(),
            items: Item::zero(),
        }
    }
}

pub trait VersionOption {
    fn value(&self) -> ComparableVersion;
}
impl VersionOption for Option<ComparableVersion> {
    fn value(&self) -> ComparableVersion {
        self.clone().unwrap_or_default()
    }
}
